"""Google Sheets app AppConfigs"""
import os

from mitol.common.apps import BaseApp


class GoogleSheetsRefundsApp(BaseApp):
    """Default configuration for the Google Sheets Refunds app"""

    name = "mitol.google_sheets_refunds"
    label = "google_sheets_refunds"
    verbose_name = "Google Sheets Refunds"

    required_settings = []

    # necessary because this is a namespaced app
    path = os.path.dirname(os.path.abspath(__file__))



"""
Parses refund request row(s), reverses orders/enrollments, and updates the spreadsheet
to reflect the processed request(s).
"""
from django.core.management import BaseCommand

from mitol.google_sheets_refunds.api import RefundRequestHandler


class Command(BaseCommand):
    """
    Parses refund request row(s), reverses orders/enrollments, and updates the spreadsheet
    to reflect the processed request(s).
    """

    help = __doc__

    def add_arguments(self, parser):
        parser.add_argument(
            "-r", "--row", type=int, help="Row number in the refund request Sheet"
        )

    def handle(self, *args, **options):
        refund_request_handler = RefundRequestHandler()
        self.stdout.write("Handling refunds and updating spreadsheet...")
        results = refund_request_handler.process_sheet(
            limit_row_index=options.get("row", None)
        )
        self.stdout.write(
            self.style.SUCCESS(
                "Refund sheet successfully processed.\n{}".format(results)
            )
        )



"""google sheets refunds settings """
from mitol.common.envs import get_int, get_string

MITOL_GOOGLE_SHEETS_REFUNDS_REQUEST_WORKSHEET_ID = get_string(
    name="MITOL_GOOGLE_SHEETS_REFUNDS_REQUEST_WORKSHEET_ID",
    default="0",
    description=(
        "ID of the worksheet within the enrollment change request spreadsheet that contains enrollment refund requests"
    ),
)
MITOL_GOOGLE_SHEETS_REFUNDS_PROCESSOR_COL = get_int(
    name="MITOL_GOOGLE_SHEETS_REFUNDS_PROCESSOR_COL",
    default=11,
    description=(
        "The zero-based index of the enrollment change sheet column that contains the user that processed the row"
    ),
)
MITOL_GOOGLE_SHEETS_REFUNDS_COMPLETED_DATE_COL = get_int(
    name="MITOL_GOOGLE_SHEETS_REFUNDS_COMPLETED_DATE_COL",
    default=13,
    description=(
        "The zero-based index of the enrollment change sheet column that contains the row completion date"
    ),
)
MITOL_GOOGLE_SHEETS_REFUNDS_ERROR_COL = get_int(
    name="MITOL_GOOGLE_SHEETS_REFUNDS_ERROR_COL",
    default=14,
    description=(
        "The zero-based index of the enrollment change sheet column that contains row processing error messages"
    ),
)
MITOL_GOOGLE_SHEETS_REFUNDS_SKIP_ROW_COL = get_int(
    name="MITOL_GOOGLE_SHEETS_REFUNDS_SKIP_ROW_COL",
    default=15,
    description=(
        "The zero-based index of the enrollment change sheet column that indicates whether the row should be skipped"
    ),
)
MITOL_GOOGLE_SHEETS_REFUNDS_FIRST_ROW = get_int(
    name="MITOL_GOOGLE_SHEETS_REFUNDS_FIRST_ROW",
    default=4,
    description=(
        "The first row (as it appears in the spreadsheet) of data that our scripts should consider "
        "processing in the refund request spreadsheet"
    ),
)

MITOL_GOOGLE_SHEETS_REFUNDS_PLUGINS = get_string(
    name="MITOL_GOOGLE_SHEETS_REFUNDS_PLUGINS",
    default="app.plugins.RefundPlugin",
    description="The path to your refund plugin",
)



"""Models for Refunds"""
from mitol.google_sheets.models import GoogleSheetsRequestModel


class RefundRequest(GoogleSheetsRequestModel):
    """Model that represents a request to refund an enrollment"""

    def __str__(self):
        return "RefundRequest: id={}, form_response_id={}, completed={}".format(
            self.id, self.form_response_id, self.date_completed is not None
        )



"""Google Sheets Refunds app util functions"""
from django.conf import settings

from mitol.google_sheets.constants import GOOGLE_API_TRUE_VAL
from mitol.google_sheets.exceptions import SheetRowParsingException
from mitol.google_sheets.utils import (
    SingletonSheetConfig,
    clean_sheet_value,
    get_column_letter,
    parse_sheet_date_only_str,
)
from mitol.google_sheets_refunds.constants import (
    SHEET_TYPE_ENROLL_CHANGE,
    WORKSHEET_TYPE_REFUND,
)


class RefundRequestRow:
    """Represents a row of the refund request sheet"""

    def __init__(
        self,
        row_index,
        response_id,
        request_date,
        learner_email,
        zendesk_ticket_no,
        requester_email,
        product_id,
        order_id,
        order_type,
        finance_email,
        finance_approve_date,
        finance_notes,
        refund_processor,
        refund_complete_date,
        errors,
        skip_row,
    ):
        self.row_index = row_index
        self.response_id = response_id
        self.request_date = request_date
        self.learner_email = learner_email
        self.zendesk_ticket_no = zendesk_ticket_no
        self.requester_email = requester_email
        self.product_id = product_id
        self.order_id = order_id
        self.order_type = order_type
        self.finance_email = finance_email
        self.finance_approve_date = finance_approve_date
        self.finance_notes = finance_notes
        self.refund_processor = refund_processor
        self.refund_complete_date = refund_complete_date
        self.errors = errors
        self.skip_row = skip_row

    @classmethod
    def parse_raw_data(cls, row_index, raw_row_data):
        """
        Parses raw row data

        Args:
            row_index (int): The row index according to the spreadsheet (not zero-based)
            raw_row_data (list of str): The raw row data

        Raises:
            SheetRowParsingException: Raised if the row could not be parsed
        """
        raw_row_data = list(map(clean_sheet_value, raw_row_data))
        try:
            return cls(
                row_index=row_index,
                response_id=int(raw_row_data[refund_sheet_config.FORM_RESPONSE_ID_COL]),
                request_date=raw_row_data[1],
                learner_email=raw_row_data[2],
                zendesk_ticket_no=raw_row_data[3],
                requester_email=raw_row_data[4],
                product_id=raw_row_data[5],
                order_id=int(raw_row_data[6]),
                order_type=raw_row_data[7],
                finance_email=raw_row_data[8],
                finance_approve_date=parse_sheet_date_only_str(raw_row_data[9]),
                finance_notes=raw_row_data[10],
                refund_processor=raw_row_data[refund_sheet_config.PROCESSOR_COL],
                refund_complete_date=parse_sheet_date_only_str(
                    raw_row_data[refund_sheet_config.COMPLETED_DATE_COL]
                ),
                errors=raw_row_data[refund_sheet_config.ERROR_COL],
                skip_row=(
                    raw_row_data[refund_sheet_config.SKIP_ROW_COL]
                    == GOOGLE_API_TRUE_VAL
                ),
            )
        except Exception as exc:
            raise SheetRowParsingException(str(exc)) from exc


class RefundRequestSheetConfig(
    SingletonSheetConfig, subclass_type=WORKSHEET_TYPE_REFUND
):
    """Metadata for the refund request spreadsheet"""

    FORM_RESPONSE_ID_COL = 0

    PROCESSOR_COL = settings.MITOL_GOOGLE_SHEETS_REFUNDS_PROCESSOR_COL
    COMPLETED_DATE_COL = settings.MITOL_GOOGLE_SHEETS_REFUNDS_COMPLETED_DATE_COL
    ERROR_COL = settings.MITOL_GOOGLE_SHEETS_REFUNDS_ERROR_COL
    SKIP_ROW_COL = settings.MITOL_GOOGLE_SHEETS_REFUNDS_SKIP_ROW_COL

    def __init__(self):
        self.sheet_type = SHEET_TYPE_ENROLL_CHANGE
        self.sheet_name = "Enrollment Change Request sheet"
        self.worksheet_type = WORKSHEET_TYPE_REFUND
        self.worksheet_name = "Refunds"
        self.first_data_row = settings.MITOL_GOOGLE_SHEETS_REFUNDS_FIRST_ROW
        self.num_columns = self.SKIP_ROW_COL + 1
        self.non_input_column_indices = set(
            # Response ID column
            [self.FORM_RESPONSE_ID_COL]
            + list(  # Every column from the finance columns to the end of the row
                range(8, self.num_columns)
            )
        )
        self.sheet_file_id = settings.MITOL_GOOGLE_SHEETS_ENROLLMENT_CHANGE_SHEET_ID

        self.PROCESSOR_COL_LETTER = get_column_letter(self.PROCESSOR_COL)
        self.ERROR_COL_LETTER = get_column_letter(self.ERROR_COL)


refund_sheet_config = RefundRequestSheetConfig()




# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: src/mitol/google_sheets_refunds:mitol-django-google-sheets-refunds

from setuptools import setup

setup(**{
    'authors': [
        'MIT Office of Open Learning <mitx-devops@mit.edu>',
    ],
    'classifiers': [
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    'description': 'Library to handle Refund requests using Google Sheets integrations in Django',
    'install_requires': (
        'django<4.0,>=2.2.12',
        'mitol-django-common~=2.5.0',
        'mitol-django-google-sheets~=2.2.6',
        'setuptools',
    ),
    'license': 'BSD 3-Clause License',
    'name': 'mitol-django-google-sheets-refunds',
    'namespace_packages': (
        'mitol',
    ),
    'package_data': {
        'mitol.google_sheets_refunds': (
            'CHANGELOG.md',
            'README.md',
        ),
    },
    'packages': (
        'mitol',
        'mitol.google_sheets_refunds',
        'mitol.google_sheets_refunds.management.commands',
        'mitol.google_sheets_refunds.migrations',
        'mitol.google_sheets_refunds.settings',
    ),
    'version': '0.3.1',
    'zip_safe': True,
})



# Generated by Django 3.2.13 on 2022-06-29 13:26

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="RefundRequest",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("updated_on", models.DateTimeField(auto_now=True)),
                ("form_response_id", models.IntegerField(db_index=True, unique=True)),
                ("date_completed", models.DateTimeField(blank=True, null=True)),
                ("raw_data", models.CharField(blank=True, max_length=300, null=True)),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



import typing

import pluggy
from django.conf import settings
from django.utils.module_loading import import_string

from mitol.google_sheets_refunds.utils import RefundRequestRow


class RefundResult(typing.NamedTuple):
    result_type: str
    message: str = None


APP_NAME = "mitol.google_sheets_refunds"

hookimpl = pluggy.HookimplMarker(APP_NAME)
hookspec = pluggy.HookspecMarker(APP_NAME)


def get_plugin_manager():
    pm = pluggy.PluginManager(APP_NAME)

    for module_path in settings.MITOL_GOOGLE_SHEETS_REFUNDS_PLUGINS:
        plugin_cls = import_string(module_path)
        pm.register(plugin_cls())

    return pm


class RefundHooks:
    @hookspec
    def refunds_process_request(self, refund_request: RefundRequestRow) -> RefundResult:
        """Hook for processing refund requests"""



"""Enrollment refund API"""
import logging

from django.conf import settings
from django.contrib.auth import get_user_model

from mitol.common.utils.datetime import now_in_utc
from mitol.google_sheets.exceptions import SheetRowParsingException
from mitol.google_sheets.sheet_handler_api import GoogleSheetsChangeRequestHandler
from mitol.google_sheets.utils import ResultType, RowResult
from mitol.google_sheets_refunds.hooks import get_plugin_manager
from mitol.google_sheets_refunds.models import RefundRequest
from mitol.google_sheets_refunds.utils import RefundRequestRow, refund_sheet_config

log = logging.getLogger(__name__)
User = get_user_model()


class RefundRequestHandler(GoogleSheetsChangeRequestHandler):
    """Manages the processing of refund requests from a spreadsheet"""

    def __init__(self):
        self.pm = get_plugin_manager()
        self.hook = self.pm.hook
        super().__init__(
            spreadsheet_id=settings.MITOL_GOOGLE_SHEETS_ENROLLMENT_CHANGE_SHEET_ID,
            worksheet_id=settings.MITOL_GOOGLE_SHEETS_REFUNDS_REQUEST_WORKSHEET_ID,
            start_row=settings.MITOL_GOOGLE_SHEETS_REFUNDS_FIRST_ROW,
            sheet_metadata=refund_sheet_config,
            request_model_cls=RefundRequest,
        )

    def process_row(self, row_index, row_data):
        """
        Ensures that the given spreadsheet row is correctly represented in the database,
        attempts to parse it, reverses/refunds the given enrollment if appropriate, and returns the
        result of processing the row.

        Args:
            row_index (int): The row index according to the spreadsheet
            row_data (list of str): The raw data of the given spreadsheet row

        Returns:
            RowResult or None: An object representing the results of processing the row, or None if
                nothing needs to be done with this row.
        """
        refund_request, request_created, request_updated = self.get_or_create_request(
            row_data
        )
        try:
            refund_req_row = RefundRequestRow.parse_raw_data(row_index, row_data)
        except SheetRowParsingException as exc:
            return RowResult(
                row_index=row_index,
                row_db_record=refund_request,
                row_object=None,
                result_type=ResultType.FAILED,
                message="Parsing failure: {}".format(str(exc)),
            )
        is_unchanged_error_row = (
            refund_req_row.errors and not request_created and not request_updated
        )
        if is_unchanged_error_row:
            return RowResult(
                row_index=row_index,
                row_db_record=refund_request,
                row_object=None,
                result_type=ResultType.IGNORED,
                message=None,
            )
        elif (
            refund_request.date_completed
            and refund_req_row.refund_complete_date is None
        ):
            return RowResult(
                row_index=row_index,
                row_db_record=refund_request,
                row_object=None,
                result_type=ResultType.OUT_OF_SYNC,
                message=None,
            )
        result_type, message = self.hook.refunds_process_request(refund_request)
        if result_type == ResultType.PROCESSED:
            refund_request.date_completed = now_in_utc()
            refund_request.save()

        return RowResult(
            row_index=row_index,
            row_db_record=refund_request,
            row_object=refund_req_row,
            result_type=result_type,
            message=message,
        )
